# Copyright 2014, Doug Wiegley (dougwig), A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import test_base

from a10_neutron_lbaas import plumbing_hooks


class TestPlumbingHooks(test_base.UnitTestBase):

    def test_select_device(self):
        hooks = plumbing_hooks.PlumbingHooks(self.a)

        a = hooks.select_device("first-token")
        hooks.select_device("second-token")
        self.assertEqual(a, hooks.select_device("first-token"))
